QUERY_STATUS_1 = 0x81
QUERY_STATUS_2 = 0x8a
QUERY_STATUS_3 = 0x8b
RESPONSE_LEN_QUERY_STATUS = 14

SET_COLOR = 0x31
RESPONSE_LEN_SET_COLOR = 1

CHANGE_MODE = 0x61
RESPONSE_LEN_CHANGE_MODE = 1

CUSTOM_MODE = 0x51
RESPONSE_LEN_CUSTOM_MODE = 0

CUSTOM_MODE_TERMINATOR_1 = 0xff
CUSTOM_MODE_TERMINATOR_2 = 0xf0

TURN_ON_1 = 0x71
TURN_ON_2 = 0x23
TURN_ON_3 = 0x0f
TURN_OFF_1 = 0x71
TURN_OFF_2 = 0x24
TURN_OFF_3 = 0x0f
RESPONSE_LEN_POWER = 4


TRUE = 0x0f
FALSE = 0xf0
ON = 0x23
OFF = 0x24
